@@ -1,5 +1,5 @@
 /* flac - Command-line FLAC encoder/decoder
- * Copyright (C) 2000,2001,2002  Josh Coalson
+ * Copyright (C) 2000,2001,2002,2003  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,16 +20,25 @@
 #define flac__decode_h
 
 #include "analyze.h"
+#include "utils.h"
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 typedef struct {
 	FLAC__bool verbose;
 	FLAC__bool continue_through_decode_errors;
 #ifdef FLAC__HAS_OGG
 	FLAC__bool is_ogg;
+	FLAC__bool use_first_serial_number;
+	long serial_number;
 #endif
-	FLAC__uint64 skip;
+	utils__SkipUntilSpecification skip_specification;
+	utils__SkipUntilSpecification until_specification;
 } decode_options_t;
 
+/* used for AIFF also */
 typedef struct {
 	decode_options_t common;
 } wav_decode_options_t;
@@ -42,6 +51,7 @@ typedef struct {
 } raw_decode_options_t;
 
 /* outfile == 0 => test only */
+int flac__decode_aiff(const char *infilename, const char *outfilename, FLAC__bool analysis_mode, analysis_options aopts, wav_decode_options_t options);
 int flac__decode_wav(const char *infilename, const char *outfilename, FLAC__bool analysis_mode, analysis_options aopts, wav_decode_options_t options);
 int flac__decode_raw(const char *infilename, const char *outfilename, FLAC__bool analysis_mode, analysis_options aopts, raw_decode_options_t options);
 
