@@ -1,5 +1,5 @@
 /* libxmms-flac - XMMS FLAC input plugin
- * Copyright (C) 2000,2001,2002  Josh Coalson
+ * Copyright (C) 2000,2001,2002,2003  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,26 +22,32 @@
 #include <stdio.h>
 #include <glib.h>
 
-#include "xmms/plugin.h"
-#include "xmms/util.h"
+#include <xmms/plugin.h>
+#include <xmms/util.h>
+#include <xmms/configfile.h>
+#include <xmms/titlestring.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+
 #include "FLAC/all.h"
+#include "plugin_common/all.h"
+#include "share/grabbag.h"
+#include "configure.h"
+#include "wrap_id3.h"
+#include "charset.h"
 
 #ifdef min
 #undef min
 #endif
 #define min(x,y) ((x)<(y)?(x):(y))
 
-typedef struct {
-	FLAC__byte raw[128];
-	char title[31];
-	char artist[31];
-	char album[31];
-	char comment[31];
-	unsigned year;
-	unsigned track; /* may be 0 if v1 (not v1.1) tag */
-	unsigned genre;
-	char description[1024]; /* the formatted description passed to xmms */
-} id3v1_struct;
+extern void FLAC_XMMS__file_info_box(char *filename);
 
 typedef struct {
 	FLAC__bool abort_flag;
@@ -53,8 +59,12 @@ typedef struct {
 	unsigned channels;
 	unsigned sample_rate;
 	unsigned length_in_msec;
+	gchar *title;
 	AFormat sample_format;
 	int seek_to_in_sec;
+	FLAC__bool has_replaygain;
+	double replay_scale;
+	DitherContext dither_context;
 } file_info_struct;
 
 static void FLAC_XMMS__init();
@@ -67,7 +77,6 @@ static int  FLAC_XMMS__get_time();
 static void FLAC_XMMS__cleanup();
 static void FLAC_XMMS__get_song_info(char *filename, char **title, int *length);
 
-static FLAC__bool get_id3v1_tag_(const char *filename, id3v1_struct *tag);
 static void *play_loop_(void *arg);
 static FLAC__bool safe_decoder_init_(const char *filename, FLAC__FileDecoder *decoder);
 static void safe_decoder_finish_(FLAC__FileDecoder *decoder);
@@ -76,15 +85,14 @@ static FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__FileDecoder *d
 static void metadata_callback_(const FLAC__FileDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data);
 static void error_callback_(const FLAC__FileDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
 
-
 InputPlugin flac_ip =
 {
 	NULL,
 	NULL,
-	"FLAC Player v" FLAC__VERSION_STRING,
+	"Reference FLAC Player v" VERSION,
 	FLAC_XMMS__init,
-	NULL,
-	NULL,
+	FLAC_XMMS__aboutbox,
+	FLAC_XMMS__configure,
 	FLAC_XMMS__is_our_file,
 	NULL,
 	FLAC_XMMS__play_file,
@@ -101,27 +109,77 @@ InputPlugin flac_ip =
 	NULL,
 	NULL,
 	FLAC_XMMS__get_song_info,
-	NULL,			/* file_info_box */
+	FLAC_XMMS__file_info_box,
 	NULL
 };
 
 #define SAMPLES_PER_WRITE 512
-static FLAC__byte reservoir_[FLAC__MAX_BLOCK_SIZE * 2 * 2 * 2]; /* *2 for max bytes-per-sample, *2 for max channels, another *2 for overflow */
-static FLAC__byte output_[FLAC__MAX_BLOCK_SIZE * 2 * 2]; /* *2 for max bytes-per-sample, *2 for max channels */
-static unsigned reservoir_samples_ = 0;
+static FLAC__int32 reservoir_[FLAC__MAX_BLOCK_SIZE * 2/*for overflow*/ * FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS];
+static FLAC__byte sample_buffer_[SAMPLES_PER_WRITE * FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS * (24/8)]; /* (24/8) for max bytes per sample */
+static unsigned wide_samples_in_reservoir_ = 0;
+
 static FLAC__FileDecoder *decoder_ = 0;
 static file_info_struct file_info_;
 static pthread_t decode_thread_;
 static FLAC__bool audio_error_ = false;
 
+#define BITRATE_HIST_SEGMENT_MSEC 500
+/* 500ms * 50 = 25s should be enough */
+#define BITRATE_HIST_SIZE 50
+static unsigned bitrate_history_[BITRATE_HIST_SIZE];
+
+
 InputPlugin *get_iplugin_info()
 {
-	flac_ip.description = g_strdup_printf("FLAC Player v%s", FLAC__VERSION_STRING);
+	flac_ip.description = g_strdup_printf("Reference FLAC Player v%s", FLAC__VERSION_STRING);
 	return &flac_ip;
 }
 
 void FLAC_XMMS__init()
 {
+	ConfigFile *cfg;
+
+	flac_cfg.title.tag_override = FALSE;
+	g_free(flac_cfg.title.tag_format);
+	flac_cfg.title.convert_char_set = FALSE;
+
+	cfg = xmms_cfg_open_default_file();
+
+	/* title */
+
+	xmms_cfg_read_boolean(cfg, "flac", "title.tag_override", &flac_cfg.title.tag_override);
+
+	if(!xmms_cfg_read_string(cfg, "flac", "title.tag_format", &flac_cfg.title.tag_format))
+		flac_cfg.title.tag_format = g_strdup("%p - %t");
+
+	xmms_cfg_read_boolean(cfg, "flac", "title.convert_char_set", &flac_cfg.title.convert_char_set);
+
+	if(!xmms_cfg_read_string(cfg, "flac", "title.file_char_set", &flac_cfg.title.file_char_set))
+		flac_cfg.title.file_char_set = FLAC_plugin__charset_get_current();
+
+	if(!xmms_cfg_read_string(cfg, "flac", "title.user_char_set", &flac_cfg.title.user_char_set))
+		flac_cfg.title.user_char_set = FLAC_plugin__charset_get_current();
+
+	/* replaygain */
+
+	xmms_cfg_read_boolean(cfg, "flac", "output.replaygain.enable", &flac_cfg.output.replaygain.enable);
+
+	xmms_cfg_read_boolean(cfg, "flac", "output.replaygain.album_mode", &flac_cfg.output.replaygain.album_mode);
+
+	if(!xmms_cfg_read_int(cfg, "flac", "output.replaygain.preamp", &flac_cfg.output.replaygain.preamp))
+		flac_cfg.output.replaygain.preamp = 0;
+
+	xmms_cfg_read_boolean(cfg, "flac", "output.replaygain.hard_limit", &flac_cfg.output.replaygain.hard_limit);
+
+	xmms_cfg_read_boolean(cfg, "flac", "output.resolution.normal.dither_24_to_16", &flac_cfg.output.resolution.normal.dither_24_to_16);
+	xmms_cfg_read_boolean(cfg, "flac", "output.resolution.replaygain.dither", &flac_cfg.output.resolution.replaygain.dither);
+
+	if(!xmms_cfg_read_int(cfg, "flac", "output.resolution.replaygain.noise_shaping", &flac_cfg.output.resolution.replaygain.noise_shaping))
+		flac_cfg.output.resolution.replaygain.noise_shaping = 1;
+
+	if(!xmms_cfg_read_int(cfg, "flac", "output.resolution.replaygain.bps_out", &flac_cfg.output.resolution.replaygain.bps_out))
+		flac_cfg.output.resolution.replaygain.bps_out = 16;
+
 	decoder_ = FLAC__file_decoder_new();
 }
 
@@ -139,14 +197,14 @@ int FLAC_XMMS__is_our_file(char *filename)
 void FLAC_XMMS__play_file(char *filename)
 {
 	FILE *f;
-	id3v1_struct tag;
 
-	reservoir_samples_ = 0;
+	wide_samples_in_reservoir_ = 0;
 	audio_error_ = false;
 	file_info_.abort_flag = false;
 	file_info_.is_playing = false;
 	file_info_.eof = false;
 	file_info_.play_thread_open = false;
+	file_info_.has_replaygain = false;
 
 	if(0 == (f = fopen(filename, "r")))
 		return;
@@ -158,6 +216,9 @@ void FLAC_XMMS__play_file(char *filename)
 	if(!safe_decoder_init_(filename, decoder_))
 		return;
 
+	if(file_info_.has_replaygain && flac_cfg.output.replaygain.enable && flac_cfg.output.resolution.replaygain.dither)
+		FLAC__plugin_common__init_dither_context(&file_info_.dither_context, file_info_.bits_per_sample, flac_cfg.output.resolution.replaygain.noise_shaping);
+
 	file_info_.is_playing = true;
 
 	if(flac_ip.output->open_audio(file_info_.sample_format, file_info_.sample_rate, file_info_.channels) == 0) {
@@ -166,8 +227,8 @@ void FLAC_XMMS__play_file(char *filename)
 		return;
 	}
 
-	(void)get_id3v1_tag_(filename, &tag);
-	flac_ip.set_info(tag.description, file_info_.length_in_msec, file_info_.sample_rate * file_info_.channels * file_info_.bits_per_sample, file_info_.sample_rate, file_info_.channels);
+	file_info_.title = flac_format_song_title(filename);
+	flac_ip.set_info(file_info_.title, file_info_.length_in_msec, file_info_.sample_rate * file_info_.channels * file_info_.bits_per_sample, file_info_.sample_rate, file_info_.channels);
 
 	file_info_.seek_to_in_sec = -1;
 	file_info_.play_thread_open = true;
@@ -219,7 +280,6 @@ void FLAC_XMMS__cleanup()
 
 void FLAC_XMMS__get_song_info(char *filename, char **title, int *length_in_msec)
 {
-	id3v1_struct tag;
 	FLAC__StreamMetadata streaminfo;
 
 	if(0 == filename)
@@ -238,9 +298,7 @@ void FLAC_XMMS__get_song_info(char *filename, char **title, int *length_in_msec)
 	}
 
 	if(title) {
-		(void)get_id3v1_tag_(filename, &tag);
-		*title = g_malloc(strlen(tag.description)+1);
-		strcpy(*title, tag.description);
+		*title = flac_format_song_title(filename);
 	}
 	if(length_in_msec)
 		*length_in_msec = streaminfo.data.stream_info.total_samples * 10 / (streaminfo.data.stream_info.sample_rate / 100);
@@ -250,88 +308,95 @@ void FLAC_XMMS__get_song_info(char *filename, char **title, int *length_in_msec)
  * local routines
  **********************************************************************/
 
-FLAC__bool get_id3v1_tag_(const char *filename, id3v1_struct *tag)
-{
-	const char *temp;
-	FILE *f = fopen(filename, "rb");
-	memset(tag, 0, sizeof(id3v1_struct));
-
-	/* set the title and description to the filename by default */
-	temp = strrchr(filename, '/');
-	if(!temp)
-		temp = filename;
-	else
-		temp++;
-	strcpy(tag->description, temp);
-	*strrchr(tag->description, '.') = '\0';
-	strncpy(tag->title, tag->description, 30); tag->title[30] = '\0';
-
-	if(0 == f)
-		return false;
-	if(-1 == fseek(f, -128, SEEK_END)) {
-		fclose(f);
-		return false;
-	}
-	if(fread(tag->raw, 1, 128, f) < 128) {
-		fclose(f);
-		return false;
-	}
-	fclose(f);
-	if(strncmp(tag->raw, "TAG", 3))
-		return false;
-	else {
-		char year_str[5];
-
-		memcpy(tag->title, tag->raw+3, 30);
-		memcpy(tag->artist, tag->raw+33, 30);
-		memcpy(tag->album, tag->raw+63, 30);
-		memcpy(year_str, tag->raw+93, 4); year_str[4] = '\0'; tag->year = atoi(year_str);
-		memcpy(tag->comment, tag->raw+97, 30);
-		tag->genre = (unsigned)((FLAC__byte)tag->raw[127]);
-		tag->track = (unsigned)((FLAC__byte)tag->raw[126]);
-
-		sprintf(tag->description, "%s - %s", tag->artist, tag->title);
-
-		return true;
-	}
-}
-
 void *play_loop_(void *arg)
 {
+	unsigned written_time_last = 0, bh_index_last_w = 0, bh_index_last_o = BITRATE_HIST_SIZE, blocksize = 1;
+	FLAC__uint64 decode_position_last = 0, decode_position_frame_last = 0, decode_position_frame = 0;
+
 	(void)arg;
 
 	while(file_info_.is_playing) {
 		if(!file_info_.eof) {
-			while(reservoir_samples_ < SAMPLES_PER_WRITE) {
+			while(wide_samples_in_reservoir_ < SAMPLES_PER_WRITE) {
+				unsigned s;
+
+				s = wide_samples_in_reservoir_;
 				if(FLAC__file_decoder_get_state(decoder_) == FLAC__FILE_DECODER_END_OF_FILE) {
 					file_info_.eof = true;
 					break;
 				}
-				else if(!FLAC__file_decoder_process_one_frame(decoder_)) {
+				else if(!FLAC__file_decoder_process_single(decoder_)) {
 					/*@@@ this should probably be a dialog */
 					fprintf(stderr, "libxmms-flac: READ ERROR processing frame\n");
 					file_info_.eof = true;
 					break;
 				}
+				blocksize = wide_samples_in_reservoir_ - s;
+				decode_position_frame_last = decode_position_frame;
+				if(!FLAC__file_decoder_get_decode_position(decoder_, &decode_position_frame))
+					decode_position_frame = 0;
 			}
-			if(reservoir_samples_ > 0) {
+			if(wide_samples_in_reservoir_ > 0) {
 				const unsigned channels = file_info_.channels;
-				const unsigned bytes_per_sample = (file_info_.bits_per_sample+7)/8;
-				unsigned i, n = min(reservoir_samples_, SAMPLES_PER_WRITE), delta;
-				unsigned bytes = n * bytes_per_sample * channels;
+				const unsigned bits_per_sample = file_info_.bits_per_sample;
+				const unsigned n = min(wide_samples_in_reservoir_, SAMPLES_PER_WRITE);
+				const unsigned delta = n * channels;
+				int bytes;
+				unsigned i, written_time, bh_index_w;
+				FLAC__uint64 decode_position;
 
-				for(i = 0; i < bytes; i++)
-					output_[i] = reservoir_[i];
-				delta = i;
-				for( ; i < reservoir_samples_*bytes_per_sample*channels; i++)
+				if(flac_cfg.output.replaygain.enable && file_info_.has_replaygain) {
+					bytes = (int)FLAC__plugin_common__apply_gain(
+						sample_buffer_,
+						reservoir_,
+						n,
+						channels,
+						bits_per_sample,
+						flac_cfg.output.resolution.replaygain.bps_out,
+						file_info_.replay_scale,
+						flac_cfg.output.replaygain.hard_limit,
+						flac_cfg.output.resolution.replaygain.dither,
+						(NoiseShaping)flac_cfg.output.resolution.replaygain.noise_shaping,
+						&file_info_.dither_context
+					);
+				}
+				else {
+					bytes = (int)FLAC__plugin_common__pack_pcm_signed_little_endian(
+						sample_buffer_,
+						reservoir_,
+						n,
+						channels,
+						bits_per_sample,
+						flac_cfg.output.resolution.normal.dither_24_to_16?
+							min(bits_per_sample, 16) :
+							bits_per_sample
+					);
+				}
+
+				for(i = delta; i < wide_samples_in_reservoir_ * channels; i++)
 					reservoir_[i-delta] = reservoir_[i];
-				reservoir_samples_ -= n;
+				wide_samples_in_reservoir_ -= n;
 
-				flac_ip.add_vis_pcm(flac_ip.output->written_time(), file_info_.sample_format, channels, bytes, output_);
+				flac_ip.add_vis_pcm(flac_ip.output->written_time(), file_info_.sample_format, channels, bytes, sample_buffer_);
 				while(flac_ip.output->buffer_free() < (int)bytes && file_info_.is_playing && file_info_.seek_to_in_sec == -1)
 					xmms_usleep(10000);
 				if(file_info_.is_playing && file_info_.seek_to_in_sec == -1)
-					flac_ip.output->write_audio(output_, bytes);
+					flac_ip.output->write_audio(sample_buffer_, bytes);
+
+				/* compute current bitrate */
+
+				written_time = flac_ip.output->written_time();
+				bh_index_w = written_time / BITRATE_HIST_SEGMENT_MSEC % BITRATE_HIST_SIZE;
+				if(bh_index_w != bh_index_last_w) {
+					bh_index_last_w = bh_index_w;
+					decode_position = decode_position_frame - (double)wide_samples_in_reservoir_ * (double)(decode_position_frame - decode_position_frame_last) / (double)blocksize;
+					bitrate_history_[(bh_index_w + BITRATE_HIST_SIZE - 1) % BITRATE_HIST_SIZE] =
+						decode_position > decode_position_last && written_time > written_time_last ?
+							8000 * (decode_position - decode_position_last) / (written_time - written_time_last) :
+							file_info_.sample_rate * file_info_.channels * file_info_.bits_per_sample;
+					decode_position_last = decode_position;
+					written_time_last = written_time;
+				}
 			}
 			else {
 				file_info_.eof = true;
@@ -345,9 +410,20 @@ void *play_loop_(void *arg)
 			unsigned target_sample = (unsigned)(distance * (double)file_info_.total_samples);
 			if(FLAC__file_decoder_seek_absolute(decoder_, (FLAC__uint64)target_sample)) {
 				flac_ip.output->flush(file_info_.seek_to_in_sec * 1000);
+				bh_index_last_w = bh_index_last_o = flac_ip.output->output_time() / BITRATE_HIST_SEGMENT_MSEC % BITRATE_HIST_SIZE;
+				if(!FLAC__file_decoder_get_decode_position(decoder_, &decode_position_frame))
+					decode_position_frame = 0;
 				file_info_.seek_to_in_sec = -1;
 				file_info_.eof = false;
-				reservoir_samples_ = 0;
+				wide_samples_in_reservoir_ = 0;
+			}
+		}
+		else {
+			/* display the right bitrate from history */
+			unsigned bh_index_o = flac_ip.output->output_time() / BITRATE_HIST_SEGMENT_MSEC % BITRATE_HIST_SIZE;
+			if(bh_index_o != bh_index_last_o && bh_index_o != bh_index_last_w && bh_index_o != (bh_index_last_w + 1) % BITRATE_HIST_SIZE) {
+				bh_index_last_o = bh_index_o;
+				flac_ip.set_info(file_info_.title, file_info_.length_in_msec, bitrate_history_[bh_index_o], file_info_.sample_rate, file_info_.channels);
 			}
 		}
 	}
@@ -358,6 +434,8 @@ void *play_loop_(void *arg)
 	flac_ip.output->buffer_free();
 	flac_ip.output->buffer_free();
 
+	g_free(file_info_.title);
+
 	pthread_exit(NULL);
 	return 0; /* to silence the compiler warning about not returning a value */
 }
@@ -371,6 +449,9 @@ FLAC__bool safe_decoder_init_(const char *filename, FLAC__FileDecoder *decoder)
 
 	FLAC__file_decoder_set_md5_checking(decoder, false);
 	FLAC__file_decoder_set_filename(decoder, filename);
+	FLAC__file_decoder_set_metadata_ignore_all(decoder);
+	FLAC__file_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_STREAMINFO);
+	FLAC__file_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_VORBIS_COMMENT);
 	FLAC__file_decoder_set_write_callback(decoder, write_callback_);
 	FLAC__file_decoder_set_metadata_callback(decoder, metadata_callback_);
 	FLAC__file_decoder_set_error_callback(decoder, error_callback_);
@@ -378,7 +459,7 @@ FLAC__bool safe_decoder_init_(const char *filename, FLAC__FileDecoder *decoder)
 	if(FLAC__file_decoder_init(decoder) != FLAC__FILE_DECODER_OK)
 		return false;
 
-	if(!FLAC__file_decoder_process_metadata(decoder))
+	if(!FLAC__file_decoder_process_until_end_of_metadata(decoder))
 		return false;
 
 	return true;
@@ -401,32 +482,19 @@ void safe_decoder_delete_(FLAC__FileDecoder *decoder)
 FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__FileDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
 {
 	file_info_struct *file_info = (file_info_struct *)client_data;
-	const unsigned bps = file_info->bits_per_sample, channels = file_info->channels, wide_samples = frame->header.blocksize;
-	unsigned wide_sample, sample, channel;
-	FLAC__int8 *scbuffer = (FLAC__int8*)reservoir_;
-	FLAC__int16 *ssbuffer = (FLAC__int16*)reservoir_;
+	const unsigned channels = file_info->channels, wide_samples = frame->header.blocksize;
+	unsigned wide_sample, offset_sample, channel;
 
 	(void)decoder;
 
 	if(file_info->abort_flag)
 		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 
-	if(bps == 8) {
-		for(sample = reservoir_samples_*channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
-			for(channel = 0; channel < channels; channel++, sample++)
-				scbuffer[sample] = (FLAC__int8)buffer[channel][wide_sample];
-	}
-	else if(bps == 16) {
-		for(sample = reservoir_samples_*channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
-			for(channel = 0; channel < channels; channel++, sample++)
-				ssbuffer[sample] = (FLAC__int16)buffer[channel][wide_sample];
-	}
-	else {
-		file_info->abort_flag = true;
-		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
-	}
+	for(offset_sample = wide_samples_in_reservoir_ * channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
+		for(channel = 0; channel < channels; channel++, offset_sample++)
+			reservoir_[offset_sample] = buffer[channel][wide_sample];
 
-	reservoir_samples_ += wide_samples;
+	wide_samples_in_reservoir_ += wide_samples;
 
 	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
 }
@@ -442,18 +510,40 @@ void metadata_callback_(const FLAC__FileDecoder *decoder, const FLAC__StreamMeta
 		file_info->channels = metadata->data.stream_info.channels;
 		file_info->sample_rate = metadata->data.stream_info.sample_rate;
 
+#ifdef FLAC__DO_DITHER
+		if(file_info->bits_per_sample == 8) {
+			file_info->sample_format = FMT_S8;
+		}
+		else if(file_info->bits_per_sample == 16 || file_info->bits_per_sample == 24) {
+			file_info->sample_format = FMT_S16_LE;
+		}
+		else {
+			/*@@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16/24-bit samples\n", "ERROR: plugin can only handle 8/16/24-bit samples", 0); */
+			file_info->abort_flag = true;
+			return;
+		}
+#else
 		if(file_info->bits_per_sample == 8) {
 			file_info->sample_format = FMT_S8;
 		}
 		else if(file_info->bits_per_sample == 16) {
-			file_info->sample_format = FMT_S16_NE;
+			file_info->sample_format = FMT_S16_LE;
 		}
 		else {
+			/*@@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16-bit samples\n", "ERROR: plugin can only handle 8/16-bit samples", 0); */
 			file_info->abort_flag = true;
 			return;
 		}
+#endif
 		file_info->length_in_msec = file_info->total_samples * 10 / (file_info->sample_rate / 100);
 	}
+	else if(metadata->type == FLAC__METADATA_TYPE_VORBIS_COMMENT) {
+		double gain, peak;
+		if(grabbag__replaygain_load_from_vorbiscomment(metadata, flac_cfg.output.replaygain.album_mode, &gain, &peak)) {
+			file_info_.has_replaygain = true;
+			file_info_.replay_scale = grabbag__replaygain_compute_scale_factor(peak, gain, (double)flac_cfg.output.replaygain.preamp, /*prevent_clipping=*/!flac_cfg.output.replaygain.hard_limit);
+		}
+	}
 }
 
 void error_callback_(const FLAC__FileDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data)
