@@ -1,5 +1,5 @@
 /* test_libFLAC - Unit tester for libFLAC
- * Copyright (C) 2000,2001,2002  Josh Coalson
+ * Copyright (C) 2000,2001,2002,2003  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -730,9 +730,17 @@ FLAC__bool test_bitbuffer()
 	}
 	printf("OK\n");
 
-	printf("testing free... OK\n");
+	printf("testing free... ");
 	FLAC__bitbuffer_free(bb);
 	FLAC__bitbuffer_free(bbcopy);
+	printf("OK\n");
+
+	printf("testing delete... ");
+	FLAC__bitbuffer_delete(bb);
+	FLAC__bitbuffer_delete(bb_zero);
+	FLAC__bitbuffer_delete(bb_one);
+	FLAC__bitbuffer_delete(bbcopy);
+	printf("OK\n");
 
 	printf("\nPASSED!\n");
 	return true;
