@@ -1,5 +1,5 @@
 /* test_libFLAC - Unit tester for libFLAC
- * Copyright (C) 2002  Josh Coalson
+ * Copyright (C) 2002,2003  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include "FLAC/file_decoder.h"
 #include "FLAC/seekable_stream_decoder.h"
 #include "FLAC/stream_decoder.h"
+#include "share/grabbag.h"
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,8 +39,8 @@ typedef struct {
 typedef stream_decoder_client_data_struct seekable_stream_decoder_client_data_struct;
 typedef stream_decoder_client_data_struct file_decoder_client_data_struct;
 
-static FLAC__StreamMetadata streaminfo_, padding_, seektable_, application1_, application2_, vorbiscomment_;
-static FLAC__StreamMetadata *expected_metadata_sequence_[6];
+static FLAC__StreamMetadata streaminfo_, padding_, seektable_, application1_, application2_, vorbiscomment_, cuesheet_, unknown_;
+static FLAC__StreamMetadata *expected_metadata_sequence_[8];
 static unsigned num_expected_;
 static const char *flacfilename_ = "metadata.flac";
 static unsigned flacfilesize_;
@@ -55,9 +56,11 @@ static FLAC__bool die_s_(const char *msg, const FLAC__StreamDecoder *decoder)
 	FLAC__StreamDecoderState state = FLAC__stream_decoder_get_state(decoder);
 
 	if(msg)
-		printf("FAILED, %s, state = %u (%s)\n", msg, (unsigned)state, FLAC__StreamDecoderStateString[state]);
+		printf("FAILED, %s", msg);
 	else
-		printf("FAILED, state = %u (%s)\n", (unsigned)state, FLAC__StreamDecoderStateString[state]);
+		printf("FAILED");
+
+	printf(", state = %u (%s)\n", (unsigned)state, FLAC__StreamDecoderStateString[state]);
 
 	return false;
 }
@@ -67,9 +70,15 @@ static FLAC__bool die_ss_(const char *msg, const FLAC__SeekableStreamDecoder *de
 	FLAC__SeekableStreamDecoderState state = FLAC__seekable_stream_decoder_get_state(decoder);
 
 	if(msg)
-		printf("FAILED, %s, state = %u (%s)\n", msg, (unsigned)state, FLAC__SeekableStreamDecoderStateString[state]);
+		printf("FAILED, %s", msg);
 	else
-		printf("FAILED, state = %u (%s)\n", (unsigned)state, FLAC__SeekableStreamDecoderStateString[state]);
+		printf("FAILED");
+
+	printf(", state = %u (%s)\n", (unsigned)state, FLAC__SeekableStreamDecoderStateString[state]);
+	if(state == FLAC__SEEKABLE_STREAM_DECODER_STREAM_DECODER_ERROR) {
+		FLAC__StreamDecoderState state_ = FLAC__seekable_stream_decoder_get_stream_decoder_state(decoder);
+		printf("      stream decoder state = %u (%s)\n", (unsigned)state_, FLAC__StreamDecoderStateString[state_]);
+	}
 
 	return false;
 }
@@ -79,113 +88,48 @@ static FLAC__bool die_f_(const char *msg, const FLAC__FileDecoder *decoder)
 	FLAC__FileDecoderState state = FLAC__file_decoder_get_state(decoder);
 
 	if(msg)
-		printf("FAILED, %s, state = %u (%s)\n", msg, (unsigned)state, FLAC__FileDecoderStateString[state]);
+		printf("FAILED, %s", msg);
 	else
-		printf("FAILED, state = %u (%s)\n", (unsigned)state, FLAC__FileDecoderStateString[state]);
+		printf("FAILED");
+
+	printf(", state = %u (%s)\n", (unsigned)state, FLAC__FileDecoderStateString[state]);
+	if(state == FLAC__FILE_DECODER_SEEKABLE_STREAM_DECODER_ERROR) {
+		FLAC__SeekableStreamDecoderState state_ = FLAC__file_decoder_get_seekable_stream_decoder_state(decoder);
+		printf("      seekable stream decoder state = %u (%s)\n", (unsigned)state, FLAC__SeekableStreamDecoderStateString[state_]);
+		if(state_ == FLAC__SEEKABLE_STREAM_DECODER_STREAM_DECODER_ERROR) {
+			FLAC__StreamDecoderState state__ = FLAC__file_decoder_get_stream_decoder_state(decoder);
+			printf("      stream decoder state = %u (%s)\n", (unsigned)state__, FLAC__StreamDecoderStateString[state__]);
+		}
+	}
 
 	return false;
 }
 
-static void *malloc_or_die_(size_t size)
-{
-	void *x = malloc(size);
-	if(0 == x) {
-		fprintf(stderr, "ERROR: out of memory allocating %u bytes\n", (unsigned)size);
-		exit(1);
-	}
-	return x;
-}
-
 static void init_metadata_blocks_()
 {
-	/*
-		most of the actual numbers and data in the blocks don't matter,
-		we just want to make sure the decoder parses them correctly
-
-		remember, the metadata interface gets tested after the decoders,
-		so we do all the metadata manipulation here without it.
-	*/
-
-	/* min/max_framesize and md5sum don't get written at first, so we have to leave them 0 */
-    streaminfo_.is_last = false;
-    streaminfo_.type = FLAC__METADATA_TYPE_STREAMINFO;
-    streaminfo_.length = FLAC__STREAM_METADATA_STREAMINFO_LENGTH;
-    streaminfo_.data.stream_info.min_blocksize = 576;
-    streaminfo_.data.stream_info.max_blocksize = 576;
-    streaminfo_.data.stream_info.min_framesize = 0;
-    streaminfo_.data.stream_info.max_framesize = 0;
-    streaminfo_.data.stream_info.sample_rate = 44100;
-    streaminfo_.data.stream_info.channels = 1;
-    streaminfo_.data.stream_info.bits_per_sample = 8;
-    streaminfo_.data.stream_info.total_samples = 0;
-	memset(streaminfo_.data.stream_info.md5sum, 0, 16);
-
-    padding_.is_last = false;
-    padding_.type = FLAC__METADATA_TYPE_PADDING;
-    padding_.length = 1234;
-
-    seektable_.is_last = false;
-    seektable_.type = FLAC__METADATA_TYPE_SEEKTABLE;
-	seektable_.data.seek_table.num_points = 2;
-    seektable_.length = seektable_.data.seek_table.num_points * FLAC__STREAM_METADATA_SEEKPOINT_LENGTH;
-	seektable_.data.seek_table.points = malloc_or_die_(seektable_.data.seek_table.num_points * sizeof(FLAC__StreamMetadata_SeekPoint));
-	seektable_.data.seek_table.points[0].sample_number = 0;
-	seektable_.data.seek_table.points[0].stream_offset = 0;
-	seektable_.data.seek_table.points[0].frame_samples = streaminfo_.data.stream_info.min_blocksize;
-	seektable_.data.seek_table.points[1].sample_number = FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER;
-	seektable_.data.seek_table.points[1].stream_offset = 1000;
-	seektable_.data.seek_table.points[1].frame_samples = streaminfo_.data.stream_info.min_blocksize;
-
-    application1_.is_last = false;
-    application1_.type = FLAC__METADATA_TYPE_APPLICATION;
-    application1_.length = 8;
-	memcpy(application1_.data.application.id, "\xfe\xdc\xba\x98", 4);
-	application1_.data.application.data = malloc_or_die_(4);
-	memcpy(application1_.data.application.data, "\xf0\xe1\xd2\xc3", 4);
-
-    application2_.is_last = false;
-    application2_.type = FLAC__METADATA_TYPE_APPLICATION;
-    application2_.length = 4;
-	memcpy(application2_.data.application.id, "\x76\x54\x32\x10", 4);
-	application2_.data.application.data = 0;
-
-    vorbiscomment_.is_last = true;
-    vorbiscomment_.type = FLAC__METADATA_TYPE_VORBIS_COMMENT;
-    vorbiscomment_.length = (4 + 8) + 4 + (4 + 5) + (4 + 0);
-	vorbiscomment_.data.vorbis_comment.vendor_string.length = 8;
-	vorbiscomment_.data.vorbis_comment.vendor_string.entry = malloc_or_die_(8);
-	memcpy(vorbiscomment_.data.vorbis_comment.vendor_string.entry, "flac 1.x", 8);
-	vorbiscomment_.data.vorbis_comment.num_comments = 2;
-	vorbiscomment_.data.vorbis_comment.comments = malloc_or_die_(vorbiscomment_.data.vorbis_comment.num_comments * sizeof(FLAC__StreamMetadata_VorbisComment_Entry));
-	vorbiscomment_.data.vorbis_comment.comments[0].length = 5;
-	vorbiscomment_.data.vorbis_comment.comments[0].entry = malloc_or_die_(5);
-	memcpy(vorbiscomment_.data.vorbis_comment.comments[0].entry, "ab=cd", 5);
-	vorbiscomment_.data.vorbis_comment.comments[1].length = 0;
-	vorbiscomment_.data.vorbis_comment.comments[1].entry = 0;
+	mutils__init_metadata_blocks(&streaminfo_, &padding_, &seektable_, &application1_, &application2_, &vorbiscomment_, &cuesheet_, &unknown_);
 }
 
 static void free_metadata_blocks_()
 {
-	free(seektable_.data.seek_table.points);
-	free(application1_.data.application.data);
-	free(vorbiscomment_.data.vorbis_comment.vendor_string.entry);
-	free(vorbiscomment_.data.vorbis_comment.comments[0].entry);
-	free(vorbiscomment_.data.vorbis_comment.comments);
+	mutils__free_metadata_blocks(&streaminfo_, &padding_, &seektable_, &application1_, &application2_, &vorbiscomment_, &cuesheet_, &unknown_);
 }
 
 static FLAC__bool generate_file_()
 {
 	printf("\n\ngenerating FLAC file for decoder tests...\n");
 
-	expected_metadata_sequence_[0] = &padding_;
-	expected_metadata_sequence_[1] = &seektable_;
-	expected_metadata_sequence_[2] = &application1_;
-	expected_metadata_sequence_[3] = &application2_;
-	expected_metadata_sequence_[4] = &vorbiscomment_;
-	num_expected_ = 5;
+	num_expected_ = 0;
+	expected_metadata_sequence_[num_expected_++] = &padding_;
+	expected_metadata_sequence_[num_expected_++] = &seektable_;
+	expected_metadata_sequence_[num_expected_++] = &application1_;
+	expected_metadata_sequence_[num_expected_++] = &application2_;
+	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_utils__generate_flacfile(flacfilename_, &flacfilesize_, 512 * 1024, &streaminfo_, expected_metadata_sequence_, num_expected_))
-		return die_("creating the encoded file"); 
+		return die_("creating the encoded file");
 
 	return true;
 }
@@ -206,21 +150,21 @@ static FLAC__StreamDecoderReadStatus stream_decoder_read_callback_(const FLAC__S
 
 	if(feof(dcd->file))
 		return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
-    else if(*bytes > 0) {
-        unsigned bytes_read = fread(buffer, 1, *bytes, dcd->file);
-        if(bytes_read == 0) {
-            if(feof(dcd->file))
-                return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
-            else
-                return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
-        }
-        else {
-            *bytes = bytes_read;
-            return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
-        }
-    }
-    else
-        return FLAC__STREAM_DECODER_READ_STATUS_ABORT; /* abort to avoid a deadlock */
+	else if(*bytes > 0) {
+		unsigned bytes_read = fread(buffer, 1, *bytes, dcd->file);
+		if(bytes_read == 0) {
+			if(feof(dcd->file))
+				return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+			else
+				return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+		}
+		else {
+			*bytes = bytes_read;
+			return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+		}
+	}
+	else
+		return FLAC__STREAM_DECODER_READ_STATUS_ABORT; /* abort to avoid a deadlock */
 }
 
 static FLAC__StreamDecoderWriteStatus stream_decoder_write_callback_(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
@@ -237,13 +181,13 @@ static FLAC__StreamDecoderWriteStatus stream_decoder_write_callback_(const FLAC_
 	if(dcd->error_occurred)
 		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 
-    if(
-        (frame->header.number_type == FLAC__FRAME_NUMBER_TYPE_FRAME_NUMBER && frame->header.number.frame_number == 0) ||
-        (frame->header.number_type == FLAC__FRAME_NUMBER_TYPE_SAMPLE_NUMBER && frame->header.number.sample_number == 0)
-    ) {
-        printf("content... ");
-        fflush(stdout);
-    }
+	if(
+		(frame->header.number_type == FLAC__FRAME_NUMBER_TYPE_FRAME_NUMBER && frame->header.number.frame_number == 0) ||
+		(frame->header.number_type == FLAC__FRAME_NUMBER_TYPE_SAMPLE_NUMBER && frame->header.number.sample_number == 0)
+	) {
+		printf("content... ");
+		fflush(stdout);
+	}
 
 	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
 }
@@ -270,7 +214,7 @@ static void stream_decoder_metadata_callback_(const FLAC__StreamDecoder *decoder
 		dcd->error_occurred = true;
 	}
 	else {
-		if(!compare_block_(expected_metadata_sequence_[dcd->current_metadata_number], metadata)) {
+		if(!mutils__compare_block(expected_metadata_sequence_[dcd->current_metadata_number], metadata)) {
 			(void)die_("metadata block mismatch");
 			dcd->error_occurred = true;
 		}
@@ -324,8 +268,8 @@ static FLAC__bool stream_decoder_test_respond_(FLAC__StreamDecoder *decoder, str
 		return false;
 	}
 
-	printf("testing FLAC__stream_decoder_process_whole_stream()... ");
-	if(!FLAC__stream_decoder_process_whole_stream(decoder))
+	printf("testing FLAC__stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__stream_decoder_process_until_end_of_stream(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 
@@ -430,13 +374,13 @@ static FLAC__bool test_stream_decoder()
 	}
 	printf("OK\n");
 
-	printf("testing FLAC__stream_decoder_process_metadata()... ");
-	if(!FLAC__stream_decoder_process_metadata(decoder))
+	printf("testing FLAC__stream_decoder_process_until_end_of_metadata()... ");
+	if(!FLAC__stream_decoder_process_until_end_of_metadata(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 
-	printf("testing FLAC__stream_decoder_process_one_frame()... ");
-	if(!FLAC__stream_decoder_process_one_frame(decoder))
+	printf("testing FLAC__stream_decoder_process_single()... ");
+	if(!FLAC__stream_decoder_process_single(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 
@@ -446,14 +390,14 @@ static FLAC__bool test_stream_decoder()
 	printf("OK\n");
 
 	decoder_client_data.ignore_errors = true;
-	printf("testing FLAC__stream_decoder_process_one_frame()... ");
-	if(!FLAC__stream_decoder_process_one_frame(decoder))
+	printf("testing FLAC__stream_decoder_process_single()... ");
+	if(!FLAC__stream_decoder_process_single(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 	decoder_client_data.ignore_errors = false;
 
-	printf("testing FLAC__stream_decoder_process_remaining_frames()... ");
-	if(!FLAC__stream_decoder_process_remaining_frames(decoder))
+	printf("testing FLAC__stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__stream_decoder_process_until_end_of_stream(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 
@@ -517,8 +461,8 @@ static FLAC__bool test_stream_decoder()
 	}
 	printf("OK\n");
 
-	printf("testing FLAC__stream_decoder_process_whole_stream()... ");
-	if(!FLAC__stream_decoder_process_whole_stream(decoder))
+	printf("testing FLAC__stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__stream_decoder_process_until_end_of_stream(decoder))
 		return die_s_("returned false", decoder);
 	printf("OK\n");
 
@@ -542,6 +486,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -580,6 +526,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -603,6 +551,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -627,6 +577,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -655,6 +607,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -771,6 +725,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -808,6 +764,8 @@ static FLAC__bool test_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	printf("testing FLAC__stream_decoder_delete()... ");
 	FLAC__stream_decoder_delete(decoder);
@@ -980,8 +938,8 @@ static FLAC__bool seekable_stream_decoder_test_respond_(FLAC__SeekableStreamDeco
 		return false;
 	}
 
-	printf("testing FLAC__seekable_stream_decoder_process_whole_stream()... ");
-	if(!FLAC__seekable_stream_decoder_process_whole_stream(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__seekable_stream_decoder_process_until_end_of_stream(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
@@ -996,6 +954,7 @@ static FLAC__bool test_seekable_stream_decoder()
 {
 	FLAC__SeekableStreamDecoder *decoder;
 	FLAC__SeekableStreamDecoderState state;
+	FLAC__StreamDecoderState sstate;
 	seekable_stream_decoder_client_data_struct decoder_client_data;
 
 	printf("\n+++ libFLAC unit test: FLAC__SeekableStreamDecoder\n\n");
@@ -1099,6 +1058,10 @@ static FLAC__bool test_seekable_stream_decoder()
 	state = FLAC__seekable_stream_decoder_get_state(decoder);
 	printf("returned state = %u (%s)... OK\n", state, FLAC__SeekableStreamDecoderStateString[state]);
 
+	printf("testing FLAC__seekable_stream_decoder_get_stream_decoder_state()... ");
+	sstate = FLAC__seekable_stream_decoder_get_stream_decoder_state(decoder);
+	printf("returned state = %u (%s)... OK\n", sstate, FLAC__StreamDecoderStateString[sstate]);
+
 	decoder_client_data.current_metadata_number = 0;
 	decoder_client_data.ignore_errors = false;
 	decoder_client_data.error_occurred = false;
@@ -1118,13 +1081,13 @@ static FLAC__bool test_seekable_stream_decoder()
 	}
 	printf("OK\n");
 
-	printf("testing FLAC__seekable_stream_decoder_process_metadata()... ");
-	if(!FLAC__seekable_stream_decoder_process_metadata(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_until_end_of_metadata()... ");
+	if(!FLAC__seekable_stream_decoder_process_until_end_of_metadata(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
-	printf("testing FLAC__seekable_stream_decoder_process_one_frame()... ");
-	if(!FLAC__seekable_stream_decoder_process_one_frame(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_single()... ");
+	if(!FLAC__seekable_stream_decoder_process_single(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
@@ -1134,8 +1097,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	printf("OK\n");
 
 	decoder_client_data.ignore_errors = true;
-	printf("testing FLAC__seekable_stream_decoder_process_one_frame()... ");
-	if(!FLAC__seekable_stream_decoder_process_one_frame(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_single()... ");
+	if(!FLAC__seekable_stream_decoder_process_single(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 	decoder_client_data.ignore_errors = false;
@@ -1145,8 +1108,8 @@ static FLAC__bool test_seekable_stream_decoder()
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
-	printf("testing FLAC__seekable_stream_decoder_process_remaining_frames()... ");
-	if(!FLAC__seekable_stream_decoder_process_remaining_frames(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__seekable_stream_decoder_process_until_end_of_stream(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
@@ -1207,8 +1170,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	}
 	printf("OK\n");
 
-	printf("testing FLAC__seekable_stream_decoder_process_whole_stream()... ");
-	if(!FLAC__seekable_stream_decoder_process_whole_stream(decoder))
+	printf("testing FLAC__seekable_stream_decoder_process_until_end_of_stream()... ");
+	if(!FLAC__seekable_stream_decoder_process_until_end_of_stream(decoder))
 		return die_ss_("returned false", decoder);
 	printf("OK\n");
 
@@ -1232,6 +1195,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1270,6 +1235,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1293,6 +1260,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1317,6 +1286,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1345,6 +1316,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1461,6 +1434,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!seekable_stream_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1498,6 +1473,8 @@ static FLAC__bool test_seekable_stream_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	printf("testing FLAC__seekable_stream_decoder_delete()... ");
 	FLAC__seekable_stream_decoder_delete(decoder);
@@ -1555,8 +1532,8 @@ static FLAC__bool file_decoder_test_respond_(FLAC__FileDecoder *decoder, file_de
 
 	dcd->current_metadata_number = 0;
 
-	printf("testing FLAC__file_decoder_process_whole_file()... ");
-	if(!FLAC__file_decoder_process_whole_file(decoder))
+	printf("testing FLAC__file_decoder_process_until_end_of_file()... ");
+	if(!FLAC__file_decoder_process_until_end_of_file(decoder))
 		return die_f_("returned false", decoder);
 	printf("OK\n");
 
@@ -1571,6 +1548,8 @@ static FLAC__bool test_file_decoder()
 {
 	FLAC__FileDecoder *decoder;
 	FLAC__FileDecoderState state;
+	FLAC__SeekableStreamDecoderState ssstate;
+	FLAC__StreamDecoderState sstate;
 	seekable_stream_decoder_client_data_struct decoder_client_data;
 
 	printf("\n+++ libFLAC unit test: FLAC__FileDecoder\n\n");
@@ -1654,6 +1633,14 @@ static FLAC__bool test_file_decoder()
 	state = FLAC__file_decoder_get_state(decoder);
 	printf("returned state = %u (%s)... OK\n", state, FLAC__FileDecoderStateString[state]);
 
+	printf("testing FLAC__file_decoder_get_seekable_stream_decoder_state()... ");
+	ssstate = FLAC__file_decoder_get_seekable_stream_decoder_state(decoder);
+	printf("returned state = %u (%s)... OK\n", ssstate, FLAC__SeekableStreamDecoderStateString[ssstate]);
+
+	printf("testing FLAC__file_decoder_get_stream_decoder_state()... ");
+	sstate = FLAC__file_decoder_get_stream_decoder_state(decoder);
+	printf("returned state = %u (%s)... OK\n", sstate, FLAC__StreamDecoderStateString[sstate]);
+
 	decoder_client_data.current_metadata_number = 0;
 	decoder_client_data.ignore_errors = false;
 	decoder_client_data.error_occurred = false;
@@ -1665,13 +1652,13 @@ static FLAC__bool test_file_decoder()
 	}
 	printf("OK\n");
 
-	printf("testing FLAC__file_decoder_process_metadata()... ");
-	if(!FLAC__file_decoder_process_metadata(decoder))
+	printf("testing FLAC__file_decoder_process_until_end_of_metadata()... ");
+	if(!FLAC__file_decoder_process_until_end_of_metadata(decoder))
 		return die_f_("returned false", decoder);
 	printf("OK\n");
 
-	printf("testing FLAC__file_decoder_process_one_frame()... ");
-	if(!FLAC__file_decoder_process_one_frame(decoder))
+	printf("testing FLAC__file_decoder_process_single()... ");
+	if(!FLAC__file_decoder_process_single(decoder))
 		return die_f_("returned false", decoder);
 	printf("OK\n");
 
@@ -1680,8 +1667,8 @@ static FLAC__bool test_file_decoder()
 		return die_f_("returned false", decoder);
 	printf("OK\n");
 
-	printf("testing FLAC__file_decoder_process_remaining_frames()... ");
-	if(!FLAC__file_decoder_process_remaining_frames(decoder))
+	printf("testing FLAC__file_decoder_process_until_end_of_file()... ");
+	if(!FLAC__file_decoder_process_until_end_of_file(decoder))
 		return die_f_("returned false", decoder);
 	printf("OK\n");
 
@@ -1748,6 +1735,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1786,6 +1775,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1809,6 +1800,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1833,6 +1826,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1861,6 +1856,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &padding_;
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -1977,6 +1974,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &seektable_;
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	if(!file_decoder_test_respond_(decoder, &decoder_client_data))
 		return false;
@@ -2014,6 +2013,8 @@ static FLAC__bool test_file_decoder()
 	expected_metadata_sequence_[num_expected_++] = &application1_;
 	expected_metadata_sequence_[num_expected_++] = &application2_;
 	expected_metadata_sequence_[num_expected_++] = &vorbiscomment_;
+	expected_metadata_sequence_[num_expected_++] = &cuesheet_;
+	expected_metadata_sequence_[num_expected_++] = &unknown_;
 
 	printf("testing FLAC__file_decoder_delete()... ");
 	FLAC__file_decoder_delete(decoder);
@@ -2039,7 +2040,7 @@ FLAC__bool test_decoders()
 	if(!test_file_decoder())
 		return false;
 
-	(void) file_utils__remove_file(flacfilename_);
+	(void) grabbag__file_remove_file(flacfilename_);
 	free_metadata_blocks_();
 
 	return true;
